hirom
;header

print " "
print "FUNCTIONS"
print "---------"

;************************************************************
;                 SRAM Variables                            |
;************************************************************
; you need 3 bytes of SRAMfor this hack and one event bit: 
; two bytes for !timer and one for !timerFlags.

; Some possibilities of SRAM to use:
;  $1CF8-$1D27: unused SRAM. SwTech names (FF6j)
;  $1E1D-$1E3F: unused SRAM. $1E20-$1E3F can be used by FF6LE+ for chest expansion.
;  $1E70-$1E7F: unused SRAM. Can be used by FF6LE and FF6LE+ for extra regular chests.

!timer = $1E1D       ; timer register ($1E1D-$1E1E)

!timerFlags = $1E1F  ; timer Flags
                     ; 0x80 = progression toward nighttime (automatically toggled)
                     ; 0x40 = timer is disabled (from event command)
                     ; 0x20 = player is currently on a regular map (automatically set on regular maps)
                     ; 0x10 = initial tint is disabled (from event command)
                     ; 0x01-0x0F = numbers of transitions toward day or night
                     
!eventByte = $1E80   ; Night event bit byte ($1E80:2)

!eventBit = $04      ; Night event bit ($1E80:2)
                     
;************************************************************
;                       Constants                           |
;************************************************************                    
!initialFlags = $88  ; initial flags values for !timerFlags
                     ; 9 transitions
                     ; flags 0x80 and 0x20 should never be set manually in a final hack
                     
!transLength = $0140 ; transition length time ($003C = 1 second)

!dayLength = $1C80   ; nighttime / daytime length ($0E10 = 1 minute)
                     ; the limit for !transLength and !dayLength ($FFFF) is between 17 and 18 minutes.
                     
!transAmount = $09   ; number of transitions (real amount is !transAmount minus 1)
                     ; limit is $0F but using the limit setting would make nighttime look black and blue only.
                     ; on the other hand, values below 4 would make nighttime and daytime look quite similar.

!nightValue = $05    ; transition ID where a current transition lower than this number
                     ; will result in a nighttime. as an example, if setting is 5, 
                     ; transition 0 to 4 will result in nighttime


;************************************************************
;                       BANK $C0                            |
;************************************************************

org $C00079             ; returning from world map
JSR setTownFlag         ; set flag telling we are on a regular map

org $C000AA             ; main map loop
JSR mainHookC0          ; manage and apply tint effect

org $C003EA             ; going into world map module
JSR clearTownFlag       ; clear flag telling we are on a regular map

org $C00468             ; returning from world map
JSR setTownFlag         ; set flag telling we are back on a regular map

org $C0C66B             ; tent event hook
JSR tentEventC0         ; switch to daytime

org $C0BE25             ; SRAM initialization
JSL initTimer           ; initialize timer related SRAM

org $C0BF67             ; map initialization
JSR initHookC0          ; save temp RAM used in bank $EE and set initial tint

org $C0992C             ; event command table
dw eventCommand         ; cycle management event command $69

;************************************************************
;                    BANK $C0 (Free Space)                  |
;************************************************************

org $C0D613

print "BANK $C0 FREE SPACE BLOCK A"
print " "

;************************************
; regular map tent function         |
;************************************
reset bytes
print "tent__C0 offset: $", pc
tentEventC0:
STA $056F               ; save tent event bank (leftover command)
JSL tentEvent           ; switch to daytime
RTS
print "tent__C0 size:    ", bytes
print " "

;************************************
; regular map flag setting function |
;************************************
reset bytes
print "set_tflg offset: $", pc
setTownFlag:            
STA $11FA               ; leftover from hook
LDA #$20                ; regular map flag value
TSB !timerFlags         ; set flag on timer flags byte
RTS
print "set_tflg size:    ", bytes
print " "

;************************************
; regular map flag clearing function|
;************************************
reset bytes
print "clr_tflg offset: $", pc
clearTownFlag:          ; clear flag telling we are on a regular map
LDA #$20                ; regular map flag value
TRB !timerFlags         ; clear flag on timer flags byte

; uncomment next 4 lines if you want to test the world map without needing 
; to restart a new game.. however if you do so make sure to save once on 
; world map then reassemble the file with those lines commented otherwise 
; the hack will not work at 100% when entering the world map from a town map.
;LDX #!transLength      ; load transition timer length 
;STX !timer             ; store as timer
;LDA #!initialFlags     ; load initial timer flags
;STA !timerFlags        ; store as flags byte

LDA $1F70,Y             ; leftover from hookup
RTS
print "clr_tflg size:    ", bytes
print " "

;************************************
; initial tint setting function     |
;************************************
reset bytes
print "init__C0 offset: $", pc
initHookC0:             ; save temp RAM used in bank $EE and set initial tint
JSR checkState          ; load map bit and verify if map is tintable or not
LDA $1C                 ; load tintability byte
BEQ +                   ; branch if it's not the case
LDX #$0009              ; number of bytes to copy to the stack ($5C-$64)               
- LDA $5B,X             ; load one bytes from RAM
PHA                     ; save to the stack  
DEX                     ; decrement loop counter                     
BNE -                   ; loop if we have not done 12 bytes
LDA !timerFlags         ; load timer flags
BIT #$10                ; check if cycle is disabled
BNE .skip               ; branch if so
JSL initTintC0          ; Set initial tint effect
.skip
LDX $00                 ; init loop counter to 0
- PLA                   ; restore one bytes from the stack
STA $5C,X               ; save in RAM ($5C-$64)
INX                     ; increment loop counter
CPX #$0009              ; check if we have done 8 bytes
BNE -                   ; loop if not
TDC                     ; clear accumulator
+ JSR $2883             ; leftover from hookup
RTS
print "init__C0 size:    ", bytes
print " "

;************************************
; regular map main timer function   |
;************************************
reset bytes
print "main__C0 offset: $", pc
mainHookC0:             ; manage and apply tint effect
JSR $0564               ; leftover from hookup
JSR checkState          ; load map bit and verify if map is tintable or not
LDA $1B                 ; load static time byte
BEQ .isNotStatic        ; branch if flag not set (static time not enabled)
RTS

.isNotStatic
LDA $1C                 ; load tintability byte
BEQ +                   ; branch if it's not the case
LDX #$000D              ; number of bytes to copy to the stack ($58-$64)               
- LDA $57,X             ; load one bytes from RAM
PHA                     ; save to the stack
DEX                     ; decrement loop counter               
BNE -                   ; loop if we have not done 13 bytes
LDA !timerFlags         ; load timer flags
BIT #$40                ; check if cycle is frozen or disabled
BNE .skip               ; branch if so
JSL decreaseTimer       ; decrement timer and apply one transtition if timer expired
.skip
LDX $00                 ; init loop counter to 0
- PLA                   ; restore one bytes from the stack
STA $58,X               ; save in RAM ($58-$64)
INX                     ; increment loop counter
CPX #$000D              ; check if we have done 13 bytes
BNE -                   ; loop if not
TDC                     ; clear accumulator
+ RTS
print "main__C0 size:    ", bytes
print " "

;************************************
; map properties check function     |
;************************************
reset bytes
print "check_st offset: $", pc
checkState:             ; load map bit and verify if map is tintable or not
REP #$20                ; 16-bit accumulator
LDA $1F64               ; load current map ID
STA $4204               ; save as divident
SEP #$20                ; 8-bit accumulator
LDA #$08                ; a byte is 8 bits
STA $4206               ; save as divider
NOP 
NOP 
NOP 
NOP 
NOP 
NOP 
NOP 
NOP                     ; wait for division to be done
LDX $4216               ; load leftover of division
LDA $C0BAFC,X           ; load bit value from LUT     
STA $1A                 ; save as temp RAM
TDC                     ; clear accumulator
LDX $4214               ; load division result (map data byte index)
LDA mapFreeze,X         ; load map freeze byte
AND $1A                 ; check if map is freezable
STA $1B                 ; save in temp RAM
LDA mapEnabling,X       ; load map tintability byte
AND $1A                 ; check if map is tintable
STA $1C                 ; save in temp RAM
RTS
print "check_st size:    ", bytes
print " "

;************************************
; battle background tint function   |
;************************************
reset bytes
print "btl_tint offset: $", pc
BattleTint:             ; tint battle background if needed
TDC                     ; clear accumulator
PHA                     ; save accumulator
PLB                     ; set DB as $00
LDA $22                 ; battle background palette ID
STA $4204               ; save as divident
STZ $4205               ; palette ID is only 1 byte
LDA #$08                ; a byte is 8 bits
STA $4206               ; save as divider
NOP 
NOP 
NOP 
NOP 
NOP 
NOP 
NOP 
NOP                     ; wait for division to be done
LDX $4216               ; load leftover of division
LDA $C0BAFC,X           ; load bit value from LUT     
STA $25                 ; save as temp RAM
TDC                     ; clear accumulator
LDX $4214               ; load division result (palette ID byte index)
LDA backgroundTint,X    ; load palette ID byte
BIT $25                 ; check if background is tintable
BEQ +                   ; branch if it's not the case
LDX #$0009              ; number of bytes to copy to the stack ($5C-$64)               
- LDA $5B,X             ; load one bytes from RAM
PHA                     ; save to the stack  
DEX                     ; decrement loop counter                     
BNE -                   ; loop if we have not done 9 bytes
LDA #$7E
PHA
PLB                     ; set DB as $7E
TDC                     ; clear accumulator
LDA $25                 ; check which of the two $C1 functions we are coming from
REP #$20                ; 16-bit accumulator
BEQ .isBattleA          ; branch if we come from $C11C36
LDA #$7EA0              ; background palette RAM offset ($7E7EA0)
BRA ++                  ; save as temporary RAM
.isBattleA
LDA #$7E60              ; background palette RAM offset ($7E7E60)
++ STA $5D              ; save as temporary RAM
LDA #$0060              ; number of bytes for loop (48 colors)
STA $5F                 ; save as temporary RAM
LDY $00                 ; clear Y index
LDX $23                 ; load palette ROM index
SEP #$20                ; 8-bit accumulator
-- LDA $E70150,X        ; load color byte
STA ($5D),Y             ; save in RAM
INX                     ; increase palette ROM index
INY                     ; increase RAM index
CPY #$0060              ; check if we have done 96 bytes
BNE --                  ; loop if not
LDY $00                 ; clear Y index
JSL getDecTintNum       ; get number of loops for initial tinting 
JSL decPals             ; tint toward nighttime multiple times
LDX $00                 ; init loop counter to 0
- PLA                   ; restore one bytes from the stack
STA $5C,X               ; save in RAM ($5D-$64)
INX                     ; increment loop counter
CPX #$0009              ; check if we have done 9 bytes
BNE -                   ; loop if not
TDC                     ; clear accumulator
+ RTL
print "btl_tint size:    ", bytes
print " "

;************************************
; battle background palette copying |
;************************************
reset bytes
print "cpy_bpal offset: $", pc
CopyBgPalRam:           ; $C11C36 loop needed separated in order to make above generic method 
REP #$20                ; 16-bit accumulator
LDX #$005F              ; 96 bytes to loop
- LDA $7EA0,X           ; palette RAM offset
STA $7CA0,X             ; save as duplicate (no idea why the game does this..)
DEX
DEX                     ; decrement loop index of 2
BNE -                   ; branch if we have not done 96 bytes
SEP #$20                ; 8-bit accumulator
RTL
print "cpy_bpal size:    ", bytes
print " "

;************************************
; cycle management event command    |
;************************************
; 69 01: freeze cycle               |
; 69 02: unfreeze cycle             |
; 69 03: disable cycle              |
; 69 04: enable cycle               |
;************************************
reset bytes
print "eventCmd offset: $", pc
eventCommand:           
LDA $EB                 ; load parameter
CMP #$01
BEQ .freezeCycle        ; branch if we freeze the cycle
CMP #$02
BEQ .unfreezeCycle      ; branch if we unfreeze the cycle
CMP #$03
BEQ .disableCycle       ; branch if we disable the cycle
CMP #$04
BEQ .enableCycle        ; branch if we enable the cycle
BRA .exit               ; invalid event command
.disableCycle
LDA #$10
TSB !timerFlags         ; disable initial tint
.freezeCycle
LDA #$40
TSB !timerFlags         ; disable timer
BRA .exit
.enableCycle
LDA #$10
TRB !timerFlags         ; enable initial tint
.unfreezeCycle
LDA #$40
TRB !timerFlags         ; enable timer
.exit
LDA #$02                ; event command is two bytes
JMP $9B5C               ; read next event command
print "eventCmd size:    ", bytes
print " "

;************************************
; cycle data                        |
;************************************
mapEnabling:           
incbin maps_enabling.bin         ; file generated with the "maps-enabling" macro in the Excel sheet
                                 ; each map has 1 bit, 1 for tint enabled, 0 for disabled
   
mapFreeze:           
incbin maps_freeze.bin           ; file generated with the "maps-freeze" macro in the Excel sheet
                                 ; each map has 1 bit, 1 for freeze enabled, 0 for disabled
     
backgroundTint:
incbin background_tints.bin      ; file generated with the "background-tints" macro in the Excel sheet
                                 ; each background has 1 bit, 1 for tint enabled, 0 for disabled

eeEvent:                 
incbin ee_events.bin             ; Data to enable day / night cycle in special scenes of world map (ee-events maro in Excel Sheet)
                                 ; 1 byte per function at $EE43DB, #$01 for enabled, #$00 for not enabled

;bgsPalTable:
;incbin background_palettes.bin  ; file generated with the "background-palettes" macro in the Excel sheet
                                 ; each background has 1 byte for nighttime palette ID (0xFF for no nighttime palette)

;************************************
; night battle background palette   |
; optional code (alpha vesion)      |
;************************************                                 
;incsrc c1PalCode.asm            ; Code needed if you have a set of night BG palettes

print "END__$C0 offset: $", pc
print " "

;************************************************************
;                       BANK $C1                            |               
;************************************************************

;************************************
; battle background tint function 1 |
;************************************
org $C11C2D
funct1C2D:
JSL checkDesert         ; check if desert background and set wavy effect if day
BRA funct1C36

org $C11C36
funct1C36:
LDA $E70000,X           ; load palette ID
AND #$7F                ; isolate palette ID
REP #$20
ASL A
ASL A
ASL A
ASL A
ASL A                   ; multiply by $20
STA $23                 ; save in temp RAM
ASL A                   ; multiply by $40
CLC 
ADC $23                 ; palette ID * $60
STA $23                 ; save in temp RAM
TAX                     ; index result
SEP #$20
LDA $E9DE
BEQ .isBg               ; branch if background
TDC
TAY
- LDA $E70150,X         ; load color byte?
STA $7E60,Y             ; save in RAM
STA $7C60,Y             ; save in RAM
INX                     ; increment color offset
INY                     ; increment RAM offset
CPY #$0040              ; have we done $40 bytes?
BNE -                   ; branch if not
BRA .exitNonBg          ; exit
.isBg
STZ $25                 ; clear temp RAM
JSL BattleTint          ; tint background if neccesary
LDA #$7E                           
PHA
PLB                     ; set DB register to $7E
JSL CopyBgPalRam        ; duplicate palette

fillbyte $ea : fill $01 ; free space (1 bytes)  
.exitNonBg
print "_$C11C77 match:  $", pc
print " "

;************************************
; battle background tint function 2 |
;************************************
org $C11DE9
funct1DE9:
JSL checkDesert         ; check if desert background and set wavy effect if day
BRA funct1DF2

org $C11DF2
funct1DF2:
LDA $E70000,X           ; load palette ID
AND #$7F                ; isolate palette ID
REP #$20
ASL A
ASL A
ASL A
ASL A
ASL A                   ; multiply by $20
STA $23                 ; save in temp RAM
ASL A                   ; multiply by $40
CLC 
ADC $23                 ; palette ID * $60
STA $23                 ; save in temp RAM
TAX
TDC
JSL BattlePalTrans      ; transfer palette prior to potential tinting
JSL BattleTint          ; tint background if neccesary
LDA #$7E
PHA
PLB                     ; set DB register to $7E
BRA +

fillbyte $ff : fill $01 ; free space (1 bytes)  
+
print "_$C11E18 match:  $", pc
print " "

;************************************
; night battle background palette   |
; optional code (alpha vesion)      |
;************************************
;org $C11C36            ; Uncomment this block if you have a set of night BG palettes
;incsrc c1Pal1C36.asm
;print "_$C11C77 $", pc

;org $C11DF2            ; Uncomment this block if you have a set of night BG palettes  
;incsrc c1Pal1DF2.asm
;print "_$C11E18 $", pc

;************************************************************
;                       BANK $EE                            |
;************************************************************

org $EE0139             ; vehicle mode hook
JSR mainHookEE          ; manage and apply tint effect   
NOP

org $EE03A6             ; character mode hook
JSR mainHookEE          ; manage and apply tint effect   
NOP

org $EE0301             ; tent event hook
JSR tentEventEE         ; switch to daytime

org $EE01E0
JSR spriteAnim          ; sprite initialization and animation

org $EE0470
JSR spriteAnim          ; sprite initialization and animation

org $EE0A41
JSR spriteAnim          ; sprite initialization and animation

org $EE0D18
JSR spriteAnim          ; sprite initialization and animation

org $EE0E62
JSR spriteAnim          ; sprite initialization and animation

org $EE156E
JSR spriteAnim          ; sprite initialization and animation

org $EE9504
JSR spriteAnim          ; sprite initialization and animation

org $EE95BD
JSR spriteAnim          ; sprite initialization and animation

org $EE96DC
JSR spriteAnim          ; sprite initialization and animation

org $EE97C5
JSR spriteAnim          ; sprite initialization and animation

org $EE98A7
JSR spriteAnim          ; sprite initialization and animation

org $EE8695             ; in airship mode initialization
JSR copyPalsB           ; load world map and objects palettes

org $EE8928             ; in chocobo mode initialization
JSR copyPalsB           ; load world map and objects palettes

org $EE8BEE             ; in character mode initialization
JSR copyPalsB           ; load world map and objects palettes

org $EE43BD
print "BANK $EE FREE SPACE BLOCK A"
print " "

;************************************
; World initital tint function      |
;************************************
reset bytes
print "init__EE offset: $", pc
initTintEE:             ; Set initial tint effect
LDA #$FF                ; full palette loop value
STA $64                 ; save as loop value
JSR loadWorldPal        ; load world maps tint parameters
JSL getDecTintNum       ; get number of loops for initial tinting 
JSL decPals             ; tint toward nighttime multiple times
JSR loadHorizonPal      ; load airship and chocobo modes horizon tint parameters
JSL decPals             ; tint toward nighttime multiple times
RTS
print "init__EE size:    ", bytes
print " "

;************************************
; palette copying main function     |
;************************************
reset bytes
print "ld_palsB offset: $", pc
copyPalsB:              ; load world map and objects palettes
JSR loadPals            ; load palettes
JSR $34D5               ; leftover from $EE hookup
RTS
print "ld_palsB size:    ", bytes

reset bytes
fillbyte $ff : fill $00 ; free space (0 bytes)
print "_filling size:    ", bytes
print " "
print "_$EE43DB match:  $", pc
print " "

org $EE84BA             ; airship mode initialization
BRA branch_850A         ; continue to $EE850A

print "BANK $EE FREE SPACE BLOCK B"
print " "

;************************************
; color tinting function (1 color)  |
; tint toward blue (night)          |
;************************************
; $5D-$5E:  active color RAM offset |
; $5F-$60:  num color bytes for loop|
; $61-$62:  temp reference color    |
;************************************
reset bytes 
print "dec__pal offset: $", pc
decPal:                 ; tint toward nighttime
LDA #$7E                ; RAM bank
PHA
PLB                     ; set DB register to $7E
-- LDA ($5D),Y          ; load byte 1 of active color
AND #$1F                ; isolate active red
STA $61                 ; save active red in temporary RAM
BEQ .exitRed            ; go to active blue if no active red present
DEC $61                 ; decrement active red value by 1
.exitRed
INC $5D                 ; increment active palette RAM address by 1
LDA ($5D),Y             ; load byte 2 of active color
AND #$7C                ; isolate active blue                  
STA $62                 ; save active blue in temporary RAM
DEC $5D                 ; decrement active palette RAM address by 1
REP #$20                ; 16-bit accumulator
LDA ($5D),Y             ; load bytes 1 and 2 of active color
AND #$03E0              ; isolate active green
BEQ .exitGreen          ; merge color channels if no active green present
SEC                     ; set carry flag for substraction
SBC #$0020              ; decrement active green value by 1
.exitGreen
ORA $61                 ; combine active green to active blue and active red
STA ($5D),Y             ; save as active RGB color
SEP #$20                ; 8-bit accumulator
INY 
INY                     ; increase color entry
CPY $5F                 ; check if we have done all colors
BNE --                  ; loop if not
TDC                     ; clear accumulator
PHA               
PLB                     ; set DB register to $00
RTL
print "dec__pal size:    ", bytes
print " "

;************************************
; world tint params loading function|
;************************************
reset bytes
print "ld_world offset: $", pc
loadWorldPal:           ; load world map tint parameters
JSR setPalLoop          ; set looping values for world map
STX $5F                 ; save as temporary RAM
LDX #$FE00              ; reference map palette RAM offset ($7EFE00)
STX $5B                 ; save result as temporary RAM
LDX #$E000              ; active map palette RAM offset ($7EE000)
STX $5D                 ; save as temporary RAM
RTS
print "ld_world size:    ", bytes

;************************************
; world tent event function         |
;************************************
reset bytes
print "tent__EE offset: $", pc
tentEventEE:
JSL tentEvent           ; switch to daytime
STZ $0205               ; clear multipurpose menu variable
RTS
print "tent__EE size:    ", bytes
print " "

reset bytes
fillbyte $ff : fill $01 ; free space (1 bytes)           
branch_850A:            ; $EE850A         
print "_filling size:    ", bytes
print " "
print "_$EE850A match:  $", pc
print " "

org $EE8767             ; chocobo mode initialization
BRA branch_87B7         ; continue to $EE87B7

print "BANK $EE FREE SPACE BLOCK C"
print " "

;************************************
; regular map palette range function|
;************************************
reset bytes
print "set_pllp offset: $", pc
setPalLoop:
LDA $64
CMP #$FF
BEQ .isInitTint         ; check if initial tint
LDA $64                 
BEQ .isFirstFrame       ; check if first frame of tint
CMP #$10
BEQ .isSecondFrame      ; check if second frame of tint
LDX #$00E0              ; third frame, colors $A0-$E0
LDY #$00A0
BRA .storeLoopVal       ; exit
.isSecondFrame
LDX #$00A0              ; second frame, colors $40-$A0
LDY #$0040
BRA .storeLoopVal       ; exit
.isInitTint
LDX #$00E0              ; initial tint, all colors
BRA .startOffset
.isFirstFrame           
LDX #$0040              ; first frame, colors $00-$40 
.startOffset
LDY $00
.storeLoopVal
RTS
print "sep_pllp size:    ", bytes
print " "

;************************************
; regular map initial tint function |
;************************************
reset bytes
print "ini_t_C0 offset: $", pc
initTintC0:             ; Set initial tint effect
LDA #$FF                ; full palette loop value
STA $64                 ; save as loop value
JSR loadTownPal         ; load regular map tint parameters
LDA #$20                ; regular map flag
TSB !timerFlags         ; set as flag
JSL getDecTintNum       ; get number of loops for initial tinting
JSL decPals             ; tint toward nighttime multiple times
RTL
print "ini_t_C0 size:    ", bytes
print " "

;************************************
; timer and flags init function     |
;************************************
reset bytes 
print "ini_time offset: $", pc
initTimer:              ; initialize timer related RAM
LDX #!transLength       ; load transition timer length
STX !timer              ; store as timer
LDA #!initialFlags      ; load initial timer flags
STA !timerFlags         ; store as flags byte
LDA $E6F566             ; leftover from $C0 hookup
RTL
print "ini_time size:    ", bytes

reset bytes
fillbyte $ff : fill $00 ; free space (0 bytes)  
branch_87B7:            ; $EE87B7
print "_filling size:    ", bytes
print " "
print "_$EE87B7 match:  $", pc
print " "

org $EE8A53             ; character mode initialization
BRA continuePalsC       ; Skip next routines

print "BANK $EE FREE SPACE BLOCK D"
print " "

;************************************
; color tinting function (1 color)  |
; tint toward day                   |
;************************************  
; $59-$5A:  new active color        |
; $5B-$5C:  ref color RAM offset    |
; $5D-$5E:  active color offset     |
; $5F-$60:  num color bytes for loop|
; $61-$62:  temp reference color    |
;************************************
reset bytes
print "inc__pal offset: $", pc
incPal:                 ; tint toward daytime
LDA #$7E                ; RAM bank
PHA
PLB                     ; set DB register to $7E
- LDA ($5B),Y           ; load byte 1 of reference color
AND #$1F                ; isolate red
BEQ .exitRed            ; exit if no red present
STA $61                 ; save in temporary RAM
LDA ($5D),Y             ; load byte 1 of active color
AND #$1F                ; isolate red
CMP #$1F                ; compare to maximum red
BEQ .exitRed            ; exit if red is at maximum
CMP $61                 ; compare to reference red
BEQ .exitRed            ; exit if active red is equal than reference red
INC A                   ; increment red value by 1
.exitRed
STA $59                 ; save in temporary color
INC $5D                 ; increment active palette RAM address by 1
LDA ($5D),Y             ; load byte 2 of active color
AND #$7C                ; isolate blue
STA $5A                 ; save in temporary color
DEC $5D                 ; decrement active palette RAM address by 1
REP #$20                ; 16-bit accumulator
LDA ($5B),Y             ; load byte 1 and 2 of reference color
AND #$03E0              ; isolate green
STA $61                 ; save in temp color
LDA ($5D),Y             ; load byte 1 and 2 of active color
AND #$03E0              ; isolate green
CMP #$03E0              ; compare to maximum green
BEQ .exitGreen          ; exit if green is at maximum
CMP $61                 ; compare to reference green
BCS .exitGreen          ; exit if active green is higher or equal than reference green
ADC #$0020              ; increase green value by 1
.exitGreen
ORA $59                 ; combine green to blue and red
STA ($5D),Y             ; save as active RGB color
SEP #$20                ; 8-bit accumulator
INY 
INY                     ; increase color entry
CPY $5F                 ; have we done all colors yet?
BNE -                   ; loop if not all colors are done
TDC                     ; clear accumulator
PHA               
PLB                     ; set DB register to $00
RTL
print "inc__pal size:    ", bytes

reset bytes    
fillbyte $ff : fill $01 ; free space (1 bytes) 
continuePalsC:          ; $EE8AA2
print "_filling size:    ", bytes
print " "
print "_$EE8AA5 match:  $", pc
print " "

;************************************************************
;              BANK $EE (Free Space)                        |
;************************************************************

org $EEAF01             ; Free bank $EE space
print "BANK $EE FREE SPACE BLOCK E"
print " "

;************************************
; world main timer function         |
;************************************
reset bytes
print "main__EE offset: $", pc
mainHookEE:             ; manage and apply tint effect
LDA !timerFlags         ; load timer flags
BIT #$40
BNE .skip   
JSL decreaseTimer       ; decrement timer and apply one transtition if timer expired
.skip
JSL $C30009             ; leftover from $EE hookup
RTS
print "main__EE size:    ", bytes
print " "

;************************************
; wavy effect check function        |
;************************************
reset bytes
print "ckdesert offset: $", pc
checkDesert:
LDA $E70000,X           ; load background palette
AND #$7F                ; isolate palette ID
CMP #$0B                ; check if wob desert palette
BEQ .check              ; branch if so
CMP #$34                ; check if wor desert palette
BEQ .check              ; branch if so
LDA $E70000,X           ; load background palette
AND #$80                ; isolate wavy bit
STA $6283               ; save it
BRA .exit               ; exit
.check               
LDA !eventByte          ; load night event bit
BIT #!eventBit
BNE .isNight            ; branch if we are the night
LDA #$80                ; wavy effect (day desert)
STA $6283               ; save it
BRA .exit
.isNight
STZ $6283               ; no wavy effect
BRA .exit
.exit
RTL
print "ckdesert size:    ", bytes
print " "

;************************************
; tent event function               |
;************************************
reset bytes
print "tentevnt offset: $", pc
tentEvent:
LDA #$80                ; Transition direction flag
TSB !timerFlags         ; set transition direction
LDA #$0F                ; load all possible transition bits
TRB !timerFlags         ; clear them
LDA #!transAmount       ; load transition amount
DEC A                   ; transition amount minus 1
TSB !timerFlags         ; set transition amount
REP #$20                ; 16-bit accumulator
LDA #!dayLength         ; load length of a day
STA !timer              ; set as timer
SEP #$20                ; 8-bit accumulator
RTL
print "tentevnt size:    ", bytes
print " "

;************************************
; battle bg palette copy function   |
;************************************
reset bytes
print "btltrans offset: $", pc
BattlePalTrans:
SEP #$20
TAY
.loop
LDA $E70150,X
STA $7EA0,Y
INX
INY
CPY #$0060
BNE .loop
TDC
TAY
LDA #$FF
STA $25
RTL
print "btltrans size:    ", bytes
print " "

;************************************
; mode 7 palette range function     |
;************************************
reset bytes
print "ld_horiz offset: $", pc
loadHorizonPal:         ; load airship and chocobo modes horizon tint parameters
LDA $64                 ; load palette loop value
CMP #$FF             
BEQ .isInitTint
LDA $64
BEQ .isFirstFrame
CMP #$10
BEQ .isSecondFrame
LDX #$0016
LDY #$000E
BRA .storeLoopVal
.isSecondFrame
LDX #$000E
LDY #$0008
BRA .storeLoopVal
.isInitTint
LDX #$0016
BRA .startOffset
.isFirstFrame           ; init starting palette offset to 2 (skip black screen border)
LDX #$0008
.startOffset
LDY $00
.storeLoopVal
STX $5F                 ; save as temporary RAM
LDX #$FF6A              ; reference map palette RAM offset ($7E7400)
STX $5B                 ; save result as temporary RAM
LDX #$E16A              ; active map palette RAM offset ($7E7400)
STX $5D                 ; save as temporary RAM
RTS
print "ld_horiz size:    ", bytes
print " "

;************************************
; numerous tint number function     |
;************************************
reset bytes
print "get_tnum offset: $", pc
getDecTintNum:          ; get number of loops for initial tinting
LDA !timerFlags         ; load timer flags
AND #$0F                ; isolate tint value
STA $63                 ; store as temporary RAM
LDA #!transAmount       ; compare to maximum tint vehicle
DEC A
SEC                     ; set carry for substraction
SBC $63                 ; substract transition value
STA $63                 ; save result in temporary RAM
RTL
print "get_tnum size:    ", bytes
print " "

;************************************
; regular map palette assignation   |
;************************************
reset bytes
print "ld_towns offset: $", pc
loadTownPal:            ; load regular map tint parameters
JSR setPalLoop
INY
INY
STX $5F                 ; save as temporary RAM
LDX #$7200              ; reference map palette RAM offset ($7E7400)
STX $5B                 ; save result as temporary RAM
LDX #$7400              ; active map palette RAM offset ($7E7400)
STX $5D                 ; save as temporary RAM
RTS
print "ld_towns size:    ", bytes
print " "

;************************************
; world events function             |
;************************************
reset bytes
print "spr_anim offset: $", pc
spriteAnim:             ; sprite initialization and animation
PHP                     ; save processor flags
PHB                     ; save data bank
REP #$30                ; 16-bit accumulator and Index
LDX $00                 ; clear X Index
- LDA $CA,X             ; load jump table event ID
BEQ +                   ; skip event if first entry
PHX                     ; save $CA-$D0 index
PHA                     ; index event ID
TAX                     ; save event ID
LDA eeEvent,X           ; load event night cycle enabling byte
BEQ .skip               ; skip timer decreasing if event has no night cycle 
SEP #$20
LDA !timerFlags         ; load timer flags
BIT #$40
BNE .skip 
JSL decreaseTimer       ; decrement timer and apply one transtition if timer expired                     ; restore event ID
.skip 
REP #$20 
PLA 
ASL A                   ; multiply by 2
TAX                     ; index jump table entry offset
JSR ($43DB,X)           ; execute animation / initialization
PLX                     ; restore $CA-$D0 index
+ INX 
INX                     ; increment $CA-$D0 index of 2
CPX #$0008              ; have we done $CA through $D0?
BNE -                   ; branch if not
PLB                     ; restore data bank
PLP                     ; restore processor flags
RTS
print "spr_anim size:    ", bytes
print " "

;************************************
; world palette loading function    |
;************************************
reset bytes
print "ld_palsA offset: $", pc
loadPals:               ; Load active and reference overworld palettes
SEP #$20    
PHB   
LDA #$7E
PHA
PLB         
LDA $001F64             ; load event byte
PHA
BNE .isNotWobA          ; check if not WOB
LDX $00                 ; WOB palettes start at $D2EC00
BRA .skipA              ; skip next instruction
.isNotWobA
LDX #$0100              ; WOR palettes start at $D2ED00     
.skipA
LDY $00                 ; init RAM counter
REP #$20                ; 16-bit accumulator    
.loopA
LDA $D2EC00,X           ; load map palette color
STA $E000,Y             ; store in active palette
STA $FE00,Y
INY
INY                     ; Increase RAM palette color entry
INX
INX                     ; increase ROM palette color entry
CPY #$0100              ; have we done 128 colors yet?
BNE .loopA              ; loop if not
SEP #$20                ; 8-bit accumulator
PLA                     ; load event byte
BNE .isNotWobB          ; check if not WOB
LDX #$0200              ; WOB palettes start at $D2EE00
BRA .skipB              ; skip next instruction             
.isNotWobB
LDX #$0300              ; WOB palettes start at $D2EF00
.skipB
LDY $00                 ; init RAM counter
REP #$20                ; 16-bit accumulator  
.loopB
LDA $D2EC00,X           ; overworld object palettes
STA $E100,Y             ; store in active palette
STA $FF00,Y
INY
INY                     ; Increase RAM palette color entry
INX
INX                     ; Increase ROM palette color entry
CPY #$0100              ; have we done 128 colors yet?
BNE .loopB              ; branch if not
PLB
SEP #$20
LDA !timerFlags         ; load timer flags
BIT #$10
BNE .skip
JSR initTintEE          ; Set initial tint effect since we just loaded palettes
.skip
REP #$20                ; 16-bit accumulator
RTS
print "ld_palsA size:    ", bytes
print " "

;************************************
; tint function (multiple times)    |
; tint toward blue (night)          |
;************************************
; $5C:      temp green ($00 to $1F) |
; $5D-$5E:  active color RAM offset |
; $5F-$60:  num color bytes for loop|
; $61-$62:  new active color        |
; $63:      tinting loop value      |
;************************************
reset bytes
print "dec_pals offset: $", pc
decPals:                ; tint toward nighttime multiple times
LDA #$7E                ; RAM bank
PHA
PLB                     ; set DB register to $7E
-- TDC                  ; clear accumulator
LDA ($5D),Y             ; load byte 1 of active color
AND #$1F                ; isolate active red
STA $61                 ; save active red in temporary RAM
BEQ .exitRed            ; go to active blue if no active red present
CMP $63                 ; compare active red value with number of loops to do 
BCC +                   ; branch if active red value is smaller than number of loops
LDA $63                 ; if equal or above, we'll use number of loops as looping value
BEQ .exitRed            ; go to active blue if number of loops equal 0
+ TAX                   ; index number of loops
- DEC $61               ; decrement active red value by 1
DEX                     ; decrement loop counter by 1
BNE -                   ; branch if loop is not finish
.exitRed
INC $5D                 ; increment active palette RAM address by 1
LDA ($5D),Y             ; load byte 2 of active color
AND #$7C                ; isolate active blue                  
STA $62                 ; save active blue in temporary RAM
DEC $5D                 ; decrement active palette RAM address by 1
REP #$20                ; 16-bit accumulator
LDA ($5D),Y             ; load byte 1 and 2 of active color
AND #$03E0              ; isolate active green
BEQ .mergeGreen         ; merge color channels if no active green present
LSR A
LSR A
LSR A
LSR A
LSR A                   ; divide by 32 (index of #$00 to #$1F)
SEP #$20                ; 8-bit accumulator
STA $5C                 ; save active green in temporary RAM
CMP $63                 ; compare active green value with number of loops to do
BCC +                   ; branch if active green value is smaller than number of loops
LDA $63                 ; if equal or above, we'll use number of loops as looping value
BEQ .exitGreen          ; restore active green and merge channels if number of loops equal 0
+ TAX                   ; index number of loops
- DEC $5C               ; decrement active green value by 1
DEX                     ; decrement loop counter by 1
BNE -                   ; branch if loop is not finish
.exitGreen
LDA $5C                 ; load active green value
REP #$20                ; 16-bit accumulator
ASL A
ASL A
ASL A
ASL A
ASL A                   ; multiply by 32 (value of #$0020 to #$0320)
.mergeGreen
ORA $61                 ; combine active green to active blue and active red
STA ($5D),Y             ; save as active RGB color
SEP #$20                ; 8-bit accumulator
INY 
INY                     ; increase color entry
CPY $5F                 ; have we done all colors yet?
BNE --                  ; loop if not all colors are done
TDC                     ; clear accumulator
PHA               
PLB                     ; set DB register to $00
RTL
print "dec_pals size:    ", bytes
print " "

;************************************
; main timer function               |
;************************************
reset bytes
print "dec_time offset: $", pc
decreaseTimer:          ; decrement timer and apply one transtition if timer expired
REP #$20                ; 16-bit accumulator
LDA !timer              ; load timer value
CMP #$0002              ; is it third last frame
BEQ .threeFramesLeft    ; branch if so
CMP #$0001              ; is it second last frame
BEQ .twoFramesLeft      ; branch if so
LDA !timer              ; load timer value
BNE .decTimer           ; decrement timer and exit if timer has not reach 0
LDA #!transLength       ; load transition timer value (relevant when passing from daytime timer to transition timer)
STA !timer              ; save as timer
BRA .isLastFrame        ; we are the last frame
.threeFramesLeft
DEC !timer              ; decrement timer
SEP #$20
LDA #$10
STA $64                 ; set third last frame flag
BRA .tint               ; tint screen
.twoFramesLeft
DEC !timer              ; decrement timer
SEP #$20
STZ $64                 ; set second last frame flag
BRA .tint               ; tint screen
.isLastFrame
print "dec_tm_A offset: $", pc
SEP #$20                ; 8-bit accumulator
LDA !timerFlags         ; load timer flags
STA $64
BIT #$0F                ; isolate transition ID
BEQ .isDarkest          ; if 0, branch to full nighttime switch
AND #$0F                ; isolate transition ID
CMP #!transAmount       ; compare to number of transitions
BEQ .isBrightest        ; if both equal, branch to full daytime switch
LDA !timerFlags         ; load timer flags
AND #$80                ; isolate transition direction
BMI .isDarker           ; branch if we are going toward nighttime
INC !timerFlags         ; increment transition ID by 1
LDA !timerFlags         ; load timer flags
AND #$0F                ; isolate transition ID
SEC
SBC #!nightValue        ; subtract night transition
BMI .tint               ; exit if night
LDA #!eventBit
TRB !eventByte          ; clear event bit
BRA .tint               ; skip next instruction
.isDarker
LDA !timerFlags         ; load timer flags
AND #$0F                ; isolate transition ID
SEC
SBC #!nightValue        ; subtract night transition
BPL .notNight           ; branch if day
LDA #!eventBit
TSB !eventByte          ; set event bit
.notNight
DEC !timerFlags         ; decrement transition ID by 1
.tint                   ; tint darker of lighter the palettes
LDA !timerFlags         ; load timer flags
BMI .decTint            ; branch if we are going toward daytime
print "dec_tm_B offset: $", pc
LDA !timerFlags         ; load timer flags
BIT #$20                ; check if we are on a regular map
BNE +                   ; exit if so
JSR loadWorldPal        ; load world maps tint parameters
JSL incPal              ; tint toward daytime
JSR loadHorizonPal      ; load mode 7 tint parameters
BRA ++
         
+ JSR loadTownPal       ; load regular map tint parameters
++ JSL incPal           ; tint toward daytime 
RTL

.decTimer
DEC A                   ; decrement timer by 1
STA !timer              ; save as new timer value
SEP #$20                ; 8-bit accumulator
RTL

print "dec_tm_C offset: $", pc
.isDarkest              ; full nighttime switch
LDA #$80                ; Transition direction flag
TRB !timerFlags         ; set next progression to be toward day
INC !timerFlags         ; increment transition ID by 1 (to avoid inifnite loop) 
BRA .setDayLength       ; set daylength timer

print "dec_tm_D offset: $", pc
.isBrightest            ; full daytime switch
LDA #$80                ; Transition direction flag
TSB !timerFlags         ; set next progression to be toward night
DEC !timerFlags         ; decrement transition ID by 1 (to avoid inifnite loop)
.setDayLength
REP #$20                ; 16-bit accumulator
LDA #!dayLength         ; load length of nighttime
STA !timer              ; set as timer
SEP #$20                ; 8-bit accumulator
RTL

print "dec_tm_E offset: $", pc
.decTint
LDA !timerFlags         ; load timer flags
BIT #$20                ; check if we are on a regular map
BNE +                   ; exit if so
JSR loadWorldPal        ; load world maps tint parameters
JSL decPal
JSR loadHorizonPal
BRA ++
                        ; skip next instruction          
+ JSR loadTownPal       ; load regular map tint parameters
++ JSL decPal           ; tint toward nighttime 
RTL
print "dec_time size:    ", bytes
print " "

print "END__$EE offset: $", pc
